package Kirjalista;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Random;
import fi.jyu.mit.ohj2.Mjonot;


/**
 * Kirja-luokka
 * - Sisältää kirjan tiedot
 * - Muuttaa merkkijonon kirjan tiedoiksi
 * - Muuttaa kirjan tiedot merkkijonoksi
 */
public class Kirja {
    private int id = 0;
    private String nimi = "";
    private String kirjoittaja  = "";
    private String kirjasarja = "";
    private int julkaisuVuosi = 0; 
    private int sivumaara = 0;
    
    private ArrayList<String> tiedot = new ArrayList<String>();
    
    private static int seuraavaId = 1;
    
        
    /**
     * Ottaa merkkijonon ja muuttaa sen kirjan tiedoiksi, poistaa turhat välilyönnit
     * merkkijono muotoa nimi|kirjoittaja|kirjasarja|sivumäärä
     * @param mjono Merkkijono, josta kirjan tiedot saadaan
     * <pre name="test">
     * Kirja k = new Kirja();
     * k.parse("Hobitti|J.R.R. Tolkien||1937|336");
     * k.toString() === "0|Hobitti|J.R.R. Tolkien||1937|336";
     * 
     * Kirja k2 = new Kirja();
     * k2.parse("    Hobitti  |  J.R.R. Tolkien |   |  1937  |  336");
     * k2.toString() === "0|Hobitti|J.R.R. Tolkien||1937|336";
     * 
     * 
     * </pre>
     */
    public void parse(String mjono) {
        StringBuilder sb = new StringBuilder(mjono);
        
        this.tiedot.removeAll(this.tiedot);
        
        this.nimi = Mjonot.erota(sb, '|').trim();
        this.kirjoittaja = Mjonot.erota(sb, '|').trim();
        this.kirjasarja = Mjonot.erota(sb, '|').trim();
        this.julkaisuVuosi = Mjonot.erota(sb, '|', 0);
        this.sivumaara = Mjonot.erotaInt(sb, 0);
        
        this.tiedot.add(this.nimi);
        this.tiedot.add(this.kirjoittaja);
        this.tiedot.add(this.kirjasarja);
        this.tiedot.add(String.valueOf(this.julkaisuVuosi));
        this.tiedot.add(String.valueOf(this.sivumaara));
    }
    
    
    /**
     * Tämä aliohjelma lukee tiedostoon tallennetut tiedot. Ei voi käyttää yllä olevaa samaan, koska id:n 
     * kanssa tulisi ongelmia
     * 
     * Ottaa merkkijonon ja muuttaa sen kirjan tiedoiksi, poistaa turhat välilyönnit
     * merkkijono muotoa id|nimi|kirjoittaja|kirjasarja|sivumäärä
     * @param mjono Merkkijono, josta kirjan tiedot saadaan
     * <pre name="test">
     * Kirja k = new Kirja();
     * k.parseTiedostosta("01|Hobitti|J.R.R. Tolkien||1937|336");
     * k.toString() === "1|Hobitti|J.R.R. Tolkien||1937|336";
     * 
     * Kirja k2 = new Kirja();
     * k2.parseTiedostosta("  2   |    Hobitti  |  J.R.R. Tolkien |   |   1937| 336");
     * k2.toString() === "2|Hobitti|J.R.R. Tolkien||1937|336";
     * 
     * 
     * </pre>
     */
    public void parseTiedostosta(String mjono) {
        StringBuilder sb = new StringBuilder(mjono);
        this.id = Mjonot.erota(sb, '|', 0);
        seuraavaId = this.id + 1;
        this.nimi = Mjonot.erota(sb, '|').trim();
        this.kirjoittaja = Mjonot.erota(sb, '|').trim();
        this.kirjasarja = Mjonot.erota(sb, '|').trim();
        this.julkaisuVuosi = Mjonot.erota(sb, '|', 0);
        this.sivumaara = Mjonot.erotaInt(sb, 0);
        
        this.tiedot.add(this.nimi);
        this.tiedot.add(this.kirjoittaja);
        this.tiedot.add(this.kirjasarja);
        this.tiedot.add(String.valueOf(this.julkaisuVuosi));
        this.tiedot.add(String.valueOf(this.sivumaara));
    }
    
    
    /**
     * Palauttaa kirjan tiedot merkkijonona muodossa id|nimi|kirjoittaja|kirjasarja|sivumäärä
     */
    @Override
    public String toString() {
        return (this.id + "|" + this.nimi + "|" + this.kirjoittaja + "|" + this.kirjasarja 
                + "|" + this.julkaisuVuosi + "|" + this.sivumaara);
    }
    
    
    /**
     * Rekisteröi kirjan antamalla sille Id:n. Kasvattaa seuraavaId:tä.
     *  <pre name="test">
     *  Kirja k = new Kirja();
     *  Kirja k2 = new Kirja();
     *  k.rekisteroi();
     *  k2.rekisteroi();
     *  k.getId() === k2.getId() - 1;
     *  </pre>
     */
    public void rekisteroi() {
        this.id = seuraavaId;
        seuraavaId++;
    }
    
    
    /**
     * Apumetodi, jolla saadaan täytettyä testiarvot kirjalle.
     */
    public void esimerkkiKirja() {
        Random rand = new Random();
        this.nimi = "Hobitti" + rand.nextInt(10);
        this.kirjoittaja = "J.R.R. Tolkien";
        this.kirjasarja = "";
        this.julkaisuVuosi = 0;
        this.sivumaara = rand.nextInt(1000);
    }

    
    /**
     * Apumetodi, jolla saadaan luotua kirjaolio testejä varten
     * @return Kirja satunnaisilla arvoilla
     */
    public static Kirja testiKirja() {
        Kirja k = new Kirja();
        k.rekisteroi();
        Random rand = new Random();
        k.nimi = "Hobitti" + rand.nextInt(10);
        k.kirjoittaja = "J.R.R. Tolkien";
        k.kirjasarja = "";
        k.julkaisuVuosi = 1937;
        k.sivumaara = rand.nextInt(1000);
        return k;
    }
    
    /**
     * @return palauttaa kirjan id:n
     */
    public int getId() {
        return this.id;
    }
    
    /**
     * @return Kirjan nimi
     */
    public String getNimi() {
        return this.nimi;
    }
    
    /**
     * Tulostaa kirjan tiedot
     * @param out Tietovirta, johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        
        String formaatti = "%s:\n"
                        + "  Kirjoittanut: %s\n"
                        + (this.kirjasarja.contentEquals("") ? "%s" : "  Kirjasarja: %s\n")
                        + "  Julkaisuvuosi: %s\n"
                        + "  Sivumäärä: %s\n";
        
        out.printf(formaatti, this.nimi, this.kirjoittaja, this.kirjasarja, this.julkaisuVuosi, this.sivumaara);
        
    }
    
    @Override
    public Kirja clone() {
        Kirja uusi = new Kirja();
        uusi.parse(this.toString());
        return uusi;
    }


    /**
     * Palauttaa olion tiedot arraylistinä
     * @return olion tiedot arraylistinä
     */
    public ArrayList<String> getTiedot() {
        return this.tiedot;
    }

}

