package Kirjalista;

import java.io.File;
import java.util.ArrayList;
import java.util.NoSuchElementException;

/**
 * Luokka, joka keskustelee käyttöliittymän kanssa 
 * - Välittää tietoja käyttöliittymän ja kirjojen sekä genrejen välillä
 */
public class Kirjasto {
    private Kirjat kirjat = new Kirjat();
    private GenreRelaatio genreRelaatio = new GenreRelaatio();
    private Genret genret = new Genret();
    
    
    
    /**
     * @param kirja Lisättävä kirja
     */
    public void lisaaKirja(Kirja kirja) {
        kirja.rekisteroi();
        kirjat.lisaa(kirja);
    }
    
    
    /**
     * Lisää genret, jos sen nimisiä ei vielä ole olemassa
     * @param genre Lisättävät genret
     */
    public void lisaaGenre(Genre[] genre) {
        for (int i = 0; i < genre.length; i++) {
            if (genret.lisaa(genre[i])) genre[i].rekisteroi();
        }
    }
    
    /**
     * Ottaa taulukollisen genrejä ja liima ne kirjan id:n kanssa yhteen
     * @param kirja Kirja
     * @param kirjanGenret Taulukko kaikista kirjan genreistä
     */
    public void lisaaGenreRelaatio(Kirja kirja, Genre[] kirjanGenret) {
        for (int i = 0; i < kirjanGenret.length; i++) {
            genreRelaatio.lisaa(new KirjanGenre(kirja, kirjanGenret[i])); 
        }
    }
    
    
    /**
     * Poistaa kirjan yhteyden genreihin
     * @param kirja Kirja
     */
    public void poistaGenreRelaatio(Kirja kirja) {
        genreRelaatio.poista(kirja.getId());
    }
    
    
    /**
     * Poistaa annetun kirjan
     * @param kirja Poistettava kirja
     */
    public void poistaKirja(Kirja kirja) {
        poistaGenreRelaatio(kirja);
        kirjat.poista(kirja.getId());
    }
    
    /**
     * @param idnro Haettavan kirjan id
     * @return Kirja annetussa indeksipaikassa
     * @throws NoSuchElementException Jos id laiton
     */
    public Kirja haeKirja(int idnro) throws NoSuchElementException {
        return kirjat.hae(idnro);
    }
    
    
    /**
     * Hakee taulukon kirjan genreistä kirjan id:n perusteella
     * @param kirjaId Sen kirjan id, jonka genrejä haetaan
     * @return Kaikki kirjan genret taulukossa
     */
    public Genre[] haeGenret(int kirjaId) {
        ArrayList<Integer> idt = genreRelaatio.haeGenret(kirjaId);
        Genre[] palautettava = new Genre[idt.size()];
        for (int i = 0; i < idt.size(); i++) {
            palautettava[i] = genret.hae(idt.get(i));
        }
        return palautettava;
    }
    
    
    /**
     * @return Kirjojen lukumäärä
     */
    public int getKirjaLkm() {
        return this.kirjat.getLkm();
    }

    
    /**
     * Hakee kirjan indeksin perusteella
     * @param ind Indeksipaikka, josta kirja haetaan
     * @return Kirja annetussa indeksipaikassa
     */
    public Kirja haeKirjaIndeksilla(int ind) {
        return kirjat.haeIndeksilla(ind);
    }

    
    /**
     * Lukee tiedostosta tiedot
     * Luo tiedostoille hakemiston, jos se puuttuu
     */
    public void lueTiedosto() {
        
        File dir = new File("tiedostot");
        dir.mkdir();
        
        this.kirjat = new Kirjat();
        this.genret = new Genret();
        this.genreRelaatio = new GenreRelaatio();
        
        kirjat.lueTiedosto();
        genret.lueTiedosto();
        genreRelaatio.lueTiedosto();
    }

    
    /**
     * Tallentaa tiedot
     */
    public void tallenna() {
       
        
        kirjat.tallenna();
        genret.tallenna();
        genreRelaatio.tallenna();
        
    }


}
