package fxKirjalista;

import java.awt.Desktop;
import java.io.IOException;
import java.io.PrintStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.ResourceBundle;

import Kirjalista.Genre;
import Kirjalista.Kirja;
import Kirjalista.Kirjasto;
import fi.jyu.mit.fxgui.ListChooser;
import fi.jyu.mit.fxgui.TextAreaOutputStream;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.ButtonBar.ButtonData;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.Pane;
import javafx.scene.control.ButtonType;
import javafx.scene.control.MenuItem;
import javafx.scene.control.SplitMenuButton;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.stage.Modality;
import javafx.stage.Stage;


/**
 * Ohjelman pääikkunan käsittelijä
 */
public class KirjalistaGUIController implements Initializable {
    
    @FXML private ListChooser<Kirja> chooserKirjat;
    @FXML private GridPane gridTiedot;

    @FXML private TextField textNimi;
    @FXML private TextField textKirjailija;
    @FXML private TextField textSarja;
    @FXML private TextField textGenre;
    @FXML private TextField textVuosi;
    @FXML private TextField textSivu;

    @FXML private TextField hakuKentta;
    @FXML private SplitMenuButton hakuehtoMenu;
    @FXML private MenuItem menuGenre;
    @FXML private MenuItem menuNimi;
    @FXML private MenuItem menuKirjailija;
    @FXML private MenuItem menuKirjasarja;
    
    /**
     * Kirjan lisäys
     */
    @FXML void handleLisaaKirja() {
        try {
            Stage lisaaStage = new Stage();
            lisaaStage.initModality(Modality.APPLICATION_MODAL);
            
            final FXMLLoader ldr = new FXMLLoader(getClass().getResource("LisaaView.fxml"));
            final Pane root = (Pane)ldr.load();
            
            final LisaaController lisaaCtrl = (LisaaController) ldr.getController();
            final Scene scene = new Scene(root);
            scene.getStylesheets().add(getClass().getResource("kirjalista.css").toExternalForm());
            lisaaStage.setScene(scene);
            lisaaStage.setTitle("Lisää kirja");
            
            lisaaCtrl.setIsanta(this);
            lisaaCtrl.setKirjasto(this.kirjasto);

            lisaaStage.show();

        } catch(Exception e) {
            e.printStackTrace();
        }

    }

    
    /**
     * Kirjan muokkaus
     */
    @FXML void handleMuokkaaKirja() {
        if (valittuKirja == null) {
            return;
        }
        try {
            Stage muokkaaStage = new Stage();
            muokkaaStage.initModality(Modality.APPLICATION_MODAL);
            
            final FXMLLoader ldr = new FXMLLoader(getClass().getResource("MuokkaaView.fxml"));
            final Pane root = (Pane)ldr.load();
            
            final LisaaController muokkaaCtrl = (LisaaController) ldr.getController();
            final Scene scene = new Scene(root);
            scene.getStylesheets().add(getClass().getResource("kirjalista.css").toExternalForm());
            muokkaaStage.setScene(scene);
            muokkaaStage.setTitle("Muokkaa kirjaa");
            
            muokkaaCtrl.setIsanta(this);
            muokkaaCtrl.setKirjasto(this.kirjasto);
 
            muokkaaCtrl.setMuokattava(valittuKirja);
            
            muokkaaStage.show();

        } catch(Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * Tietojen tallentaminen
     */
    @FXML void handleTallenna() {
        tallenna();
    }

    /**
     * Apua-valikon tietoja-valinta
     */
    @FXML void handleTietoja() {
        tietoja();
    }


    /**
     * Tiedosto-valikon tulosta-valinta
     */
    @FXML void handleTulosta() {
        TulostusController ctrl = TulostusController.tulosta(null);
        tulosta(ctrl.getTextArea());
    }


    /**
     * Muokkaa-valikon poista-valinta
     */
    @FXML void poistaKirja() {
         poistoVahvistus();
    }
    
    @FXML void handleHaku() {
        hae(0);
    }

    


/*============================================================================*/    
    
    private Kirjasto kirjasto;
    private Kirja valittuKirja;
    private String hakuEhto = "nimi";
    
    
    /**
     * Vaihtaa hakuehdon valittuun
     * @param valittu Valittu hakuehto
     */
    public void hakuEhto(MenuItem valittu) {
        String ehto = valittu.getText();
        if (hakuehtoMenu.getText().toLowerCase().contentEquals(ehto.toLowerCase())) {
            hakuehtoMenu.setText("Hakuehto");
            this.hakuEhto = "nimi";
            hae(0);
            return;
        }
        hakuehtoMenu.setText(ehto);
        this.hakuEhto = ehto.toLowerCase();
        hae(0);
    }
    
    /**
     * Luo näytölle dialogin, joka kysyy vahvistusta kirjan poistamiseksi, jos 
     * vastaus on kyllä, kirja poistetaan
     */
    public void poistoVahvistus() {
        Alert alert = new Alert(Alert.AlertType.CONFIRMATION);
        alert.setTitle("Poista");
        alert.setHeaderText(null);
        alert.setContentText("Haluatko varmasti poistaa kirjan?");

        ButtonType buttonTypeYes = new ButtonType("Kyllä", ButtonData.OK_DONE);
        ButtonType buttonTypeCancel = new ButtonType("Ei", ButtonData.CANCEL_CLOSE);

        alert.getButtonTypes().setAll(buttonTypeYes, buttonTypeCancel);

        Optional<ButtonType> result = alert.showAndWait();
        
        if ( result.get() == buttonTypeYes ) {
            kirjasto.poistaKirja(valittuKirja);
            hae(0);
        }
    }

    
    /**
     * Asettaa controllerille viitteen kirjasto-olioon
     * @param kirjasto Kirjasto-luokan olio, johon viite asetetaan
     */
    public void setKirjasto(Kirjasto kirjasto) {
        this.kirjasto = kirjasto;
    }
    
    
    /**
     * Hakee kirjat listchooseriin hakuehdon perusteella.
     * Näyttää (yleensä) ylimmän kirjan tiedot.
     * @param indeksi yleensä 0, paitsi jos halutaan näyttää tietyn kirjan tiedot
     */
    public void hae(int indeksi) {
        chooserKirjat.clear();
        
        String haettuTeksti = hakuKentta.getText().toLowerCase();
        int hakuIndeksi = 0;
        
        // TODO voisi tehdä paremmin
        switch (hakuEhto) {
            case "genre":
                hakuIndeksi = -1;
                break;
            case "nimi":
                hakuIndeksi = 0;
                break;
            case "kirjailija":
                hakuIndeksi = 1;
                break;                
            case "kirjasarja":
                hakuIndeksi = 2;
                break;                
            default:
                return;
        }
        
        
        // Haku muille ehdoille kuin genreille
        if (hakuIndeksi >= 0) {       
            for (int i = 0; i < kirjasto.getKirjaLkm(); i++) {
                
                Kirja kirja = kirjasto.haeKirjaIndeksilla(i);
                ArrayList<String> kentat = kirja.getTiedot();
                
                if (kentat.get(hakuIndeksi).toLowerCase().contains(haettuTeksti)) {
                    chooserKirjat.add(kirja.getNimi(), kirja);
                }
                    
            }
        } else { // Haku, jos hakuehto on genret
            for (int i = 0; i < kirjasto.getKirjaLkm(); i++) {
                
                Kirja kirja = kirjasto.haeKirjaIndeksilla(i);
                Genre[] genret = kirjasto.haeGenret(kirja.getId());
                
                for (Genre g : genret) {
                    if (g.getNimi().contains(haettuTeksti)) {
                        chooserKirjat.add(kirja.getNimi(), kirja);
                        break;
                    }
                }
                    
            }
        }
            
            
        chooserKirjat.setSelectedIndex(indeksi);
        naytaJasen();
    }
    
    
    /**
     * Näyttää listasta valitun kirjan tiedot
     */
    protected void naytaJasen() {
        valittuKirja = chooserKirjat.getSelectedObject();

        if (valittuKirja == null) return;

        TextField[] tekstit = { textNimi, textKirjailija, textSarja, textVuosi, textSivu };
        ArrayList<String> tiedot = valittuKirja.getTiedot();
        
        for (int i = 0; i < tekstit.length; i++) {
            tekstit[i].setText(tiedot.get(i));
        }
        
        StringBuilder genreText = new StringBuilder("");
        
        for (Genre g : kirjasto.haeGenret(valittuKirja.getId())) {
            genreText.append(g.getNimi());
            genreText.append(", ");
        }
        
        genreText.delete(genreText.length() - 2, genreText.length());
        
        textGenre.setText(genreText.toString());
        
    }
    
    

    /**
     * Tulostaa hakuehdon täyttävien kirjojen tiedot textareaan
     * @param teksti textarea
     */
    public void tulosta(TextArea teksti) {
        List<Kirja> kirjat = chooserKirjat.getObjects();
        
        try (PrintStream os = TextAreaOutputStream.getTextPrintStream(teksti)) {
            for (Kirja k: kirjat) { 
                k.tulosta(os);
                Genre[] genret = kirjasto.haeGenret(k.getId());
                if (genret.length == 1) {
                    os.println("  Genre:");
                } else os.println("  Genret:");
                for (Genre g : genret) {
                    os.print("      -");
                    os.println(g.getNimi());
                }
                os.println();
            }
        }
        
    }
    
    
    /**
     * Tekee tarvittavat muut alustukset
     */
    protected void alusta() {
        chooserKirjat.clear();
        chooserKirjat.addSelectionListener(_ -> naytaJasen());
        
        // TODO voisi tehdä paremmin
        menuGenre.setOnAction( _ -> hakuEhto(menuGenre) );
        menuNimi.setOnAction( _ -> hakuEhto(menuNimi) );
        menuKirjailija.setOnAction( _ -> hakuEhto(menuKirjailija) );
        menuKirjasarja.setOnAction( _ -> hakuEhto(menuKirjasarja) );
        
        
    }

    @Override
    public void initialize(URL arg0, ResourceBundle arg1) {
        alusta();
        
    }


    /**
     * Ohjelman käynnistyessä lukee tiedostosta tiedot ja asettaa ne näytölle
     */
    public void lueTiedosto() {
        kirjasto.lueTiedosto();
        hae(0);
    }

    
    /**
     * Kirjoittaa tiedot tiedostoihin
     */
    public void tallenna() {
        kirjasto.tallenna();
    }


    /**
     * Tarkistetaan onko tallennus tehty
     * @return true, jos sovelluksen saa sulkea
     */
    public boolean voikoSulkea() {
        tallenna();
        return true;
    }


    /**
     * @return Valittu kirja
     */
    public Kirja getValittuKirja() {
        return this.valittuKirja;
    }

    
    /**
     * Näytetään ohjelman suunnitelma erillisessä selaimessa.
     */
    public void tietoja() { 
        Desktop desktop = Desktop.getDesktop();
        try {
            URI uri = new URI("https://tim.jyu.fi/view/kurssit/tie/ohj2/v/2025/kevat/ht/tornhale");
            desktop.browse(uri);
        } catch (URISyntaxException e) {
            return;
        } catch (IOException e) {
            return;
        }
    

        
    }
    
}
