package fxKirjalista;

import java.util.ArrayList;

import Kirjalista.Genre;
import Kirjalista.Kirja;
import Kirjalista.Kirjasto;
import javafx.fxml.FXML;
import javafx.scene.control.TextField;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ModalController;


/**
 * Kirjan liäämisikkunan käsittelijä
 */
public class LisaaController {

  
    @FXML private TextField textGenre;
    @FXML private TextField textKirjailija;
    @FXML private TextField textNimi;
    @FXML private TextField textSarja;
    @FXML private TextField textSivu;
    @FXML private TextField textVuosi;

    
    private Kirjasto kirjasto;
    
    private KirjalistaGUIController isanta;
    
    @FXML void handleLisaaOK() {
        uusiKirja();
    }


    @FXML void handleMuokkaaOK() {
        TextField[] tekstit = { textNimi, textKirjailija, textSarja, textVuosi, textSivu };
        
        if (tekstit[0].getText().isEmpty()) {
            Dialogs.showMessageDialog("Et antanut kirjan nimeä");
            return;
        }
        
        if (tekstit[1].getText().isEmpty()) {
            Dialogs.showMessageDialog("Et antanut kirjailijan nimeä");
            return;
        }
        
        muokkaaKirja();
        ModalController.closeStage(textNimi);
    }


    @FXML void handlePoista() {
        isanta.poistoVahvistus();
        ModalController.closeStage(textNimi);

    }
    
    
    /**
     * Asettaa controllerille viitteen kirjasto-olioon
     * @param kirjasto Kirjasto-luokan olio, johon viite asetetaan
     */
    public void setKirjasto(Kirjasto kirjasto) {
        this.kirjasto = kirjasto;
    }
    
    
    
    /**
     * Asettaa olion "isännäksi" kirjalistan pääikkunan kontrollerin, jotta
     * keskustelu kontrollerien välillä toimii
     * @param isanta Pääikkunan kontrolleri
     */
    public void setIsanta(KirjalistaGUIController isanta) {
        this.isanta = isanta;
    }
    
    
    /**
     * Luo uuden kirjan, genret ja relaatiot. 
     */
    protected void uusiKirja() {
        
        TextField[] tekstit = { textNimi, textKirjailija, textSarja, textVuosi, textSivu };
        
        if (tekstit[0].getText().isEmpty()) {
            Dialogs.showMessageDialog("Et antanut kirjan nimeä");
            return;
        }
        
        if (tekstit[1].getText().isEmpty()) {
            Dialogs.showMessageDialog("Et antanut kirjailijan nimeä");
            return;
        }
        
        Kirja uusi = new Kirja();
        
        StringBuilder sb = new StringBuilder();
        for (TextField tf : tekstit) {
            sb.append(tf.getText());
            sb.append("|");
        }
        sb.deleteCharAt(sb.length() - 1);
        
        uusi.parse(sb.toString());
        
        
        String[] genreStr = textGenre.getText().split(",");
        Genre[] genret = new Genre[genreStr.length];
        for (int i = 0; i < genreStr.length; i++) {
            Genre g = new Genre();
            g.parse(genreStr[i]);
            genret[i] = g;
        }
        kirjasto.lisaaGenre(genret);
        
        kirjasto.lisaaGenreRelaatio(uusi, genret);
        
        kirjasto.lisaaGenre(genret);
        
        kirjasto.lisaaKirja(uusi);
        
        kirjasto.lisaaGenreRelaatio(uusi, genret);
        
        isanta.hae(kirjasto.getKirjaLkm() - 1);

        
        ModalController.closeStage(textNimi);
    }


    
    /**
     * Asettaa muokattavan kirjan tiedot muokkausikkunaan
     * @param muokattava Muokattava kirja
     */
    public void setMuokattava(Kirja muokattava) {
        TextField[] tekstit = { textNimi, textKirjailija, textSarja, textVuosi, textSivu };
        
        ArrayList<String> tiedot = muokattava.getTiedot();
        
        for (int i = 0; i < tekstit.length; i++) {
            tekstit[i].setText(tiedot.get(i));
        }
        
        StringBuilder genreText = new StringBuilder("");
        
        for (Genre g : kirjasto.haeGenret(muokattava.getId())) {
            genreText.append(g.getNimi());
            genreText.append(", ");
        }
        genreText.delete(genreText.length() - 2, genreText.length());
        
        textGenre.setText(genreText.toString());    
        
    }

    
    /**
     * Muokkaa kirjan
     */
    public void muokkaaKirja() {
        TextField[] tekstit = { textNimi, textKirjailija, textSarja, textVuosi, textSivu };
        
        Kirja muokattava = this.isanta.getValittuKirja();
        
        StringBuilder sb = new StringBuilder();
        for (TextField tf : tekstit) {
            sb.append(tf.getText());
            sb.append("|");
        }
        sb.deleteCharAt(sb.length() - 1);
        
        muokattava.parse(sb.toString());
        
        String[] genreStr = textGenre.getText().split(",");
        Genre[] genret = new Genre[genreStr.length];
        for (int i = 0; i < genreStr.length; i++) {
            Genre g = new Genre();
            g.parse(genreStr[i]);
            genret[i] = g;
        }
        kirjasto.lisaaGenre(genret);
                
        kirjasto.poistaGenreRelaatio(muokattava);
        kirjasto.lisaaGenreRelaatio(muokattava, genret);
        
        this.isanta.naytaJasen();
    }
    
}
